@@ -89,7 +89,7 @@ exc_descriptors[]=
     {SQLITE_MISMATCH, "Mismatch"},
 
     /* Internal Errors */
-    {SQLITE_INTERNAL, "Internal"},
+    {SQLITE_INTERNAL, "Internal"},  /* NOT USED */
     {SQLITE_PROTOCOL, "Protocol"},
     {SQLITE_MISUSE,   "Misuse"},
     {SQLITE_RANGE,    "Range"},
@@ -113,7 +113,7 @@ exc_descriptors[]=
     {SQLITE_IOERR,    "IO"},
     {SQLITE_CORRUPT,  "Corrupt"},
     {SQLITE_FULL,     "Full"},
-    {SQLITE_TOOBIG,   "TooBig"},
+    {SQLITE_TOOBIG,   "TooBig"},     /* NOT USED */
     {SQLITE_NOLFS,    "NoLFS"},
     {SQLITE_EMPTY,    "Empty"},
     {SQLITE_FORMAT,   "Format"},
@@ -196,7 +196,7 @@ static void make_exception(int res, sqlite3 *db)
 
 /* CALLBACK INFO */
 
-/* details of a registered function passed as user data to aqlite3_create_function */
+/* details of a registered function passed as user data to sqlite3_create_function */
 typedef struct _funccbinfo 
 {
   struct _funccbinfo *next;       /* we use a linked list */
@@ -1276,6 +1276,12 @@ getaggregatefunctioncontext(sqlite3_context *context)
 }
 
 
+/*
+  Note that we can't call sqlite3_result_error in the step function as
+  SQLite doesn't want to you to do that (and core dumps!)
+  Consequently if an error is returned, we will still be repeatedly
+  called.
+*/
 
 static void
 cbdispatch_step(sqlite3_context *context, int argc, sqlite3_value **argv)
@@ -1287,14 +1293,19 @@ cbdispatch_step(sqlite3_context *context, int argc, sqlite3_value **argv)
 
   gilstate=PyGILState_Ensure();
 
+  if (PyErr_Occurred())
+    goto finally;
+
   aggfc=getaggregatefunctioncontext(context);
 
+  if (PyErr_Occurred())
+    goto finally;
+
   assert(aggfc);
   
   if(aggfc->aggvalue==Py_None)
     {
       assert(PyErr_Occurred());
-      sqlite3_result_error(context, "Prior Python Error", SQLITE_ERROR);
       goto finally;
     }
 
@@ -1308,7 +1319,9 @@ cbdispatch_step(sqlite3_context *context, int argc, sqlite3_value **argv)
   Py_XDECREF(retval);
 
   if(!retval)
-    sqlite3_result_error(context, "Error in step function", SQLITE_ERROR);
+    {
+      assert(PyErr_Occurred());
+    }
 
  finally:
    PyGILState_Release(gilstate);
@@ -1330,7 +1343,7 @@ cbdispatch_final(sqlite3_context *context)
 
   assert(aggfc);
   
-  if(!aggfc->finalfunc)
+  if(PyErr_Occurred() || !aggfc->finalfunc)
     {
       sqlite3_result_error(context, "Prior Python Error", SQLITE_ERROR);
       goto finally;
@@ -2114,16 +2127,18 @@ Cursor_dobindings(Cursor *self)
   /* verify the number of args supplied */
   if (self->bindings)
     sz=PySequence_Fast_GET_SIZE(self->bindings);
+  /* there is another statement after this one ... */
   if(*self->zsqlnextpos && sz-self->bindingsoffset<nargs)
     {
       PyErr_Format(ExcBindings, "Incorrect number of bindings supplied.  The current statement uses %d and there are only %d left.  Current offset is %d",
-                   nargs, PySequence_Fast_GET_SIZE(self->bindings), self->bindingsoffset);
+                   nargs, (self->bindings)?sz:0, self->bindingsoffset);
       return -1;
     }
+  /* no more statements */
   if(!*self->zsqlnextpos && sz-self->bindingsoffset!=nargs)
     {
       PyErr_Format(ExcBindings, "Incorrect number of bindings supplied.  The current statement uses %d and there are %d supplied.  Current offset is %d",
-                   nargs, PySequence_Fast_GET_SIZE(self->bindings), self->bindingsoffset);
+                   nargs, (self->bindings)?sz:0, self->bindingsoffset);
       return -1;
     }
   
